#ifndef DDF_SESSION_PAYLOADTERMINATOR_H
#define DDF_SESSION_PAYLOADTERMINATOR_H

// *******************************************************************
// NOTICE:
//
//      Copyright (C) Bloomberg L.P., 2002
//      All Rights Reserved.
//      Property of Bloomberg L.P. (BLP)
//      This software is made available solely pursuant to the
//      terms of a BLP license agreement which governs its use.
// *******************************************************************
//
// Create Date: 08/09/2002
//        File: PayloadTerminator.h
//     Creator: Chad Elliott
//     Purpose: This file contains the declaration(s) for the
//                 following class:
//                    - PayloadTerminator
//
//
// *******************************************************************

// *******************************************************************
// Include files.
// *******************************************************************

#include "ddf/Export.h"
#include "ddf/Transport/Callback.h"
#include "ddf/Session/SessionHeader.h"
// Start Shallow Cache/Slow Consumer
#include "ddf/Transport/DataStreamCommon.h"
#include "ddf/Session/PayloadAcceptor.h"
// End Shallow Cache/Slow Consumer 

#include <string>
// Start Shallow Cache/Slow Consumer
#include <map>
// End Shallow Cache/Slow Consumer 

// *******************************************************************
// Forward declarations.
// *******************************************************************

class ACE_Message_Block;

namespace DDF
{

  // *******************************************************************
  // Forward declarations.
  // *******************************************************************

  class AdminMessageGeneratorBase;
  
  namespace Logging
  {
    class Logger;
  }

  namespace Session
  {
    // *******************************************************************
    // Forward declarations.
    // *******************************************************************

    class DataStreamTermination;

    // *****************************************************************
    // Class declaration.
    // *****************************************************************

    class DDF_Export PayloadTerminator
    {
    public:
      /**
       * Destructor.
       */
      ~PayloadTerminator();

      /**
       * Get the admin callback.
       */
      Transport::Callback* getAdminCallback();

      /**
       * Get the data callback.
       */
      Transport::Callback* getDataCallback();

      /**
       * Reset the sequence number checking back to 1.
       */
      void resetSequenceNumbers();
      //Change to fix BUG-20: Overriding this method here to reset only admin sequence nos. 
      void resetAdminSequenceNumbers();


      // Start Shallow Cache/Slow Consumer
      int getSequenceNumbers(Transport::ConnectionType type, 
          SequenceNumber*& sequenceNumbers);
      // End Shallow Cache/Slow Consumer 

      // Start Shallow Cache/Slow Consumer 
    //private:
      /**
       * Only the PayloadTerminatorFactory should construct one of these.
       */
     // friend class PayloadTerminatorFactory;

      /**
       * Constructor.
       *
       * @param dso The owning DataStreamTermination.
       * @param logger The logger with which we log errors.
       */
      // Start Shallow Cache/Slow Consumer 
      // PayloadTerminator(DataStreamTermination* dso,
      // End Shallow Cache/Slow Consumer 
      PayloadTerminator(PayloadAcceptor* pa,
                        AdminMessageGeneratorBase*& adminMessageGenerator,
                        const std::string& serviceName,
                        Logging::Logger* logger);

    private:
      /**
       * Copy constructor is not implemented.
       */
      PayloadTerminator(const PayloadTerminator& rhs);

      /**
       * Assignment operator is not implemented.
       */
      PayloadTerminator& operator=(const PayloadTerminator& rhs);

      class SequenceChecker {
      public:
        // Start Shallow Cache/Slow Consumer 
        //SequenceChecker(DataStreamTermination* dst,
        SequenceChecker(PayloadAcceptor* pa,
        // End Shallow Cache/Slow Consumer 
                        AdminMessageGeneratorBase*& adminMessageGenerator,
                        const std::string& serviceName,
                        Logging::Logger* logger);
        virtual ~SequenceChecker();
        // Start Shallow Cache/Slow Consumer 
        // void checkSequenceNumber(ACE_Message_Block* mb);
        SequenceNumber checkSequenceNumber(ACE_Message_Block *mb);
        int getSequenceNumbers(SequenceNumber*& sequenceNumber);
        // End Shallow Cache/Slow Consumer 
        void resetSequenceNumber();
        //Change to fix BUG-20: Overriding this method here to reset only admin sequence nos. 
        void resetAdminSequenceNumber();

      protected:
        // Start Shallow Cache/Slow Consumer 
        // DataStreamTermination* dst_;
        PayloadAcceptor* payloadAcceptor_;
        // End Shallow Cache/Slow Consumer 


      private:
        // Start Shallow Cache/Slow Consumer 
        //MessageSequenceNumber sequenceNumber_;
        struct PublisherSequenceInfo {
          MessageSequenceNumber messageSequenceNumber;
          bool sequenceStarted;
        };

        typedef std::map<PublisherIdentifier, 
                         PublisherSequenceInfo*> PublisherMap;
        typedef std::map<PublisherIdentifier, 
                         PublisherSequenceInfo*>::const_iterator  
                         PublisherMapItr;

        PublisherMap sequenceMap_;
        //bool sequenceStarted_;
        // End Shallow Cache/Slow Consumer 
        AdminMessageGeneratorBase*& adminMessageGenerator_;
        std::string serviceName_;
        Logging::Logger* logger_;
      };

    class AdminCallback: public Transport::Callback,
                         public SequenceChecker
      {
      public:
        // Start Shallow Cache/Slow Consumer 
        // AdminCallback(DataStreamTermination* dst,
        AdminCallback(PayloadAcceptor* pa,
            // End Shallow Cache/Slow Consumer 
            AdminMessageGeneratorBase*& adminMessageGenerator,
            const std::string& serviceName,
            Logging::Logger* logger);
        virtual void initiate(ACE_Message_Block* mb);
      };

    class DataCallback: public Transport::Callback,
                        public SequenceChecker
      {
      public:
        // Start Shallow Cache/Slow Consumer
        // DataCallback(DataStreamTermination* dst,
        DataCallback(PayloadAcceptor* pa,
            // End Shallow Cache/Slow Consumer 
            AdminMessageGeneratorBase*& adminMessageGenerator,
            const std::string& serviceName,
            Logging::Logger* logger);
        virtual void initiate(ACE_Message_Block* mb);
      };

      AdminCallback adminCallback_;
      DataCallback dataCallback_;
    };
  }
}

#endif
